#include <stdint.h>
#include <stdbool.h>
#include "ble.h"
#include "ble_srv_common.h"
#include "boards.h"
#include "nrf_gpio.h"
#include "app_mpu.h"

//2da39e80-3a63-11e9-b210-d663bd873d93
//f364adc9-b000-4042-ba50-05ca45bf8abc

//B9B63980-42EB-4A82-99C0-6926637C1207

#define BLE_SRM_SERVICE_BASE_UUID                                                                      \
    {                                                                                                  \
        0x07, 0x12, 0x7C, 0x63, 0x26, 0x69, 0xC0, 0x99, 0x82, 0x4A, 0xeB, 0x42, 0x80, 0x39, 0xB6, 0xB9 \
    }

#define BLE_SRM_SERVICE_UUID    0x3980
#define BLE_SRM_STATE_CHAR_UUID 0x3981
#define BLE_SRM_CMD_CHAR_UUID   0x3982
#define BLE_SRM_DATA_CHAR_UUID  0x398A

/**@brief   Macro for defining a ble_cus instance.
 *
 * @param   _name   Name of the instance.
 * @hideinitializer
 */
#define BLE_SRVC_OPT_DEF(_name)                     \
    static ble_srvc_opt_t _name;                    \
    NRF_SDH_BLE_OBSERVER(_name##_obs,               \
                         BLE_HRS_BLE_OBSERVER_PRIO, \
                         ble_srvc_opt_on_ble_evt, &_name)

typedef enum
{
    BLE_SRVC_EVT_STATE_NOTIFICATION_ENABLED,   /**< Custom value notification enabled event. */
    BLE_SRVC_EVT_STATE_NOTIFICATION_DISABLED,  /**< Custom value notification disabled event. */
    BLE_SRVC_EVT_STATE_CHAR_WRITE,       /**< State Write notification. */
    BLE_SRVC_EVT_CMD_CHAR_WRITE,         /**< Command Write notification. */
    BLE_SRVC_EVT_DISCONNECTED,
    BLE_SRVC_EVT_CONNECTED
} ble_srvc_opt_evt_type_t;

/**@brief Custom Service event. */
typedef struct
{
    ble_srvc_opt_evt_type_t evt_type; /**< Type of event. */
} ble_srvc_opt_evt_t;
// Forward declaration of the ble_srvc_opt_t type.
typedef struct ble_srvc_opt_s ble_srvc_opt_t;

/**@brief Custom Service event handler type. */
typedef void (*ble_srvc_opt_evt_handler_t)(ble_srvc_opt_t *p_cus, ble_srvc_opt_evt_t *p_evt);

/**@brief Custom Service init structure. This contains all options and data needed for
 *        initialization of the service.*/
typedef struct
{
    ble_srvc_opt_evt_handler_t evt_handler;                 /**< Event handler to be called for handling events in the Custom Service. */
    uint8_t initial_custom_value;                           /**< Initial custom value */
    ble_srv_cccd_security_mode_t custom_value_char_attr_md; /**< Initial security level for Custom characteristics attribute */
} ble_srvc_opt_init_t;

/**@brief Custom Service structure. This contains various status information for the service. */
struct ble_srvc_opt_s
{

    ble_srvc_opt_evt_handler_t evt_handler;         /**< Event handler to be called for handling events in the Custom Service. */
    uint16_t service_handle;                        /**< Handle of Custom Service (as provided by the BLE stack). */
    ble_gatts_char_handles_t state_char_handle;     /**< State Characteristic Handle. */
    ble_gatts_char_handles_t cmd_char_handle;       /**< Command Characteristic Handle. */
    ble_gatts_char_handles_t data_char_handle;      /**< Data Characteristic Handle. */
    uint16_t conn_handle;                           /**< Handle of the current connection (as provided by the BLE stack, is BLE_CONN_HANDLE_INVALID if not in a connection). */
    uint8_t uuid_type;
};

typedef struct
{
    int16_t serial;
    int16_t x;
    int16_t y;
    int16_t z;
}data_values_t;

/**@brief Function for handling the Application's BLE Stack events.
 *
 * @details Handles all events from the BLE stack of interest to the Battery Service.
 *
 * @note 
 *
 * @param[in]   p_ble_evt  Event received from the BLE stack.
 * @param[in]   p_context  Custom Service structure.
 */
void ble_srvc_opt_on_ble_evt(ble_evt_t const *p_ble_evt, void *p_context);

/**@brief Function for updating the custom value.
 *
 * @details The application calls this function when the cutom value should be updated. If
 *          notification has been enabled, the custom value characteristic is sent to the client.
 *
 * @note 
 *       
 * @param[in]   p_cus          Custom Service structure.
 * @param[in]   Custom value 
 *
 * @return      NRF_SUCCESS on success, otherwise an error code.
 */
uint32_t ble_srvc_opt_custom_value_update(ble_srvc_opt_t *p_cus, uint8_t custom_value);

uint32_t ble_srvc_opt_init(ble_srvc_opt_t *p_cus, const ble_srvc_opt_init_t *p_srvc_opt_init);

//For sending a State
uint32_t ble_srvc_set_state(ble_srvc_opt_t *p_service, uint8_t state_value);

uint32_t ble_mpu_update(ble_srvc_opt_t *p_cus, accel_values_t * accel_values);